class WritingConference
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::MultiParameterAttributes

  include OrbitModel::LanguageRestrict
  include OrbitModel::Status
  include OrbitTag::Taggable

  LANGUAGE_TYPES = [ "English", "Chinese" ]

  field :paper_title, localize: true
  field :conference_title, localize: true
  field :authors, localize: true
  field :location, localize: true
  field :sponsor, localize: true

  # has_and_belongs_to_many :tags, :class_name => "PersonalConferenceTag"
  has_and_belongs_to_many :conference_author_types
  has_and_belongs_to_many :conference_paper_types
  has_and_belongs_to_many :conference_co_authors

  field :year
  field :language
  field :period_start_date, :type => Date
  field :period_end_date, :type => Date
  field :keywords
  field :abstract
  field :publication_date, :type => Date
  field :url
  field :note
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId

  paginates_per 10

  has_many :writing_conference_files, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :writing_conference_files, :allow_destroy => true

  # before_save :clean_checkboxs

  validates :paper_title, :at_least_one => true
  before_validation :add_http
  after_save :save_writing_conference_files
  validates :url, :format => /^(http|https):\/\/(([a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5})|((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))(:[0-9]{1,5})?(\/.*)?/i, :unless => Proc.new{self.url.blank?}

  attr_reader :author_tokens

  def author_tokens=(ids)
    authors_ids = ids.split(",").map{|id|
      begin
        ConferenceCoAuthor.find(id).id
      rescue
        if id != "0"
          new_co_author = ConferenceCoAuthor.new(:co_author => id)
          new_co_author.save
          new_co_author.id
        else
          id
        end
      end
    }
    self.conference_co_author_ids = authors_ids
  end

  def self.search( category_id = nil )
    if category_id.to_s.size > 0
      find(:all, :conditions => {writing_conference_category_id: category_id}).desc( :is_top, :title )
    else
      find(:all).desc( :is_top, :title)
    end
  end

  def self.widget_datas
    where( :is_hidden => false ).desc(:is_top, :created_at)
  end

  def save_writing_conference_files
    self.writing_conference_files.each do |t|
      if t.should_destroy
        t.destroy
      end
    end
  end

  def create_link
    title = ["#{self.paper_title}"]
    title << self.conference_title if self.conference_title.present?
    title << self.sponsor if self.sponsor.present?
    title << self.location if self.location.present?
    title << "#{period_start_date}-#{period_end_date}" if (self.period_start_date.present? && self.period_end_date.present?)
    # title << "(#{self.conference_paper_types.collect{|x| x.title}.join(', ')})"
    title.join(', ') 
  end

  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end

  def clean_checkboxs
    self.tagged_ids.delete('')
    self.conference_author_type_ids.delete('')
    self.conference_level_type_ids.delete('')
  end

end
